/*
* Copyright (c) 2017 SecureRF Corporation, All rights reserved.
* -------------------------------------------------------------------------
* This computer program is proprietary information of SecureRF Corporation.
* The receipt or possession of this program does not convey any rights to
* reproduce or disclose its contents, or to manufacture, use, or sell
* anything that it may describe, in whole or in part, except for the purposes of 
* the post-quantum algorithm public review and evaluation in connection with 
* the National Institute of Standards and Technology’s development of standards 
* for post-quantum cryptography.  Any reproduction of this program for any other 
* purpose without the express written consent of SecureRF is a violation of the 
* copyright laws and may subject you to criminal prosecution.
* -------------------------------------------------------------------------
*/
#ifdef KAT_INTERMEDIATES

#include <stdio.h>
#include "aelib.h"
#include "printUtils.h"
#include "ae_crypto.h"
#include "ae_dsa.h"
#include <arpa/inet.h>

void printBraid(unsigned char *braid,  char *name)
{
	printf("%s:\n",name);

	int sig_length = ((((braid[0] << 8) | braid[1]) + 1)/2) + 2;
	int i = 0, count = 0;
	for(i=0; i<sig_length; i++)
	{
        count++;
		printf("%02x,\t", braid[i]);
        if(count == 8)
        {
            printf("\n");
            count = 0;
        }   
	}	
	printf("\n");

	fflush(stdout);
}

void printplist(ae_plist_t braid,  char *name)
{
	printf("%s:\n",name);

	int i = 0, count = 0;
	for(i=0; i<braid->len; i++)
	{
        count++;
		if(braid->b[i].u == -1)
			printf("-");

		printf("%d\t", braid->b[i].l - 1);
        if(count == 8)
        {
            printf("\n");
            count = 0;
        }
        
            
	}	
	printf("\n");

	fflush(stdout);
}

void print_column_major_matrix(ffelem matrix[WALNUT_BRAID][WALNUT_MATRIX_ROW],  char *name)
{
	int i, j;
	printf("%s\n", name );

	for(i=0; i<WALNUT_BRAID; i++) {
	        for(j=0; j<WALNUT_BRAID; j++) {

				printf("%"STRFMT",\t", matrix[j][i]);
			}
			printf("\n");
	}

	printf("\n");
}

void unpack_and_print_matrix(GF_T *matrix,  char *name)
{
	int i,j, count = 0;

	printf("%s\n", name);

	printf("%"STRFMT"\t", NTOH(matrix[count]));
	count++;

	printf("\nT-values:\n");

	for(i=0; i<WALNUT_BRAID; i++) {
		printf("%"STRFMT",\t", NTOH(matrix[count]));
		count++;
	}

	printf("\nMatrix:\n");

	for(i=0; i<WALNUT_BRAID-1; i++) {
	        for(j=0; j<WALNUT_BRAID; j++) {
	        	printf("%"STRFMT",\t", NTOH(matrix[count]));
	    		count++;
	        }
	        printf("\n");
	}

	for(i=0; i<WALNUT_BRAID-1; i++) {
		printf("%"STRFMT",\t", 0);
	} 
	printf("%08x\nMatrix Prime\n", NTOH(matrix[count]));
	count++;


	for(i=0; i<WALNUT_BRAID-1; i++) {
		for(j=0; j<WALNUT_BRAID; j++) {
			printf("%"STRFMT",\t", NTOH(matrix[count]));
			count++;
		}
		printf("\n");
	}

	for(i=0; i<WALNUT_BRAID-1; i++) {
			printf("%"STRFMT",\t", 0);
		}
	printf("%"STRFMT"\nPermutation:\n", NTOH(matrix[count]));
	count++;

	AePackCtx pack;
	ae_pack_init(&pack, &matrix[count], PACKED_PERMUTATION_LENGTH, 4);

	for (i = 0; i < WALNUT_BRAID; i++)
	{
		printf("%02x, ", ae_unpack_entry(&pack));
	}

	printf("\n");
}

void printBuffer(void *braid, size_t size,  char *name)
{
	printf("%s:\n",name);

	GF_T *b = braid;

	int i = 0, count = 0;
	for(i=0; i<size; i++)
	{
        count++;
		printf("%"STRFMT",\t", b[i]);
        if(count == WALNUT_BRAID)
        {
            printf("\n");
            count = 0;
        }
	}
	printf("\n");
}

void printCharBuffer(unsigned char *braid, size_t size,  char *name)
{
	printf("%s:\n",name);

	int i = 0, count = 0;
	for(i=0; i<size; i++)
	{
        count++;
		printf("%02x,\t", braid[i]);
        if(count == 8)
        {
            printf("\n");
            count = 0;
        }
	}	
	printf("\n");
}
#endif
