/*
* Copyright (c) 2017 SecureRF Corporation, All rights reserved.
* -------------------------------------------------------------------------
* This computer program is proprietary information of SecureRF Corporation.
* The receipt or possession of this program does not convey any rights to
* reproduce or disclose its contents, or to manufacture, use, or sell
* anything that it may describe, in whole or in part, except for the purposes of
* the post-quantum algorithm public review and evaluation in connection with
* the National Institute of Standards and Technology’s development of standards
* for post-quantum cryptography.  Any reproduction of this program for any other
* purpose without the express written consent of SecureRF is a violation of the
* copyright laws and may subject you to criminal prosecution.
* -------------------------------------------------------------------------
*/

#ifndef AE_DH_H
#define AE_DH_H

#include <stdint.h>
#include "aelib.h"
#include <stddef.h>

#ifndef GTC_BYTES
#       define GTC_BYTES 8
#endif

#if     GTC_BYTES == 1
#       define  MATRIX_ELEM     uint8_t
#elif   GTC_BYTES == 2
#       define  MATRIX_ELEM     uint16_t
#elif   GTC_BYTES == 4
#       define  MATRIX_ELEM     uint32_t
#elif   GTC_BYTES == 8
#       define  MATRIX_ELEM     uint64_t
#else
#       error Invalid definition of GTC_BYTES
#endif
#define WALNUT_LENGTH_BIT 14

#ifndef WALNUT_MATRIX_ROW
#	define WALNUT_MATRIX_ROW	12
#endif

typedef struct walnut_verify_ctx {
	union {
		ffelem ans2[WALNUT_BRAID][WALNUT_MATRIX_ROW];
		ffelem generators[(4 * 22) / 2];
	} d;
	ffelem encode[WALNUT_BRAID][WALNUT_MATRIX_ROW];

	union {
		ffelem ans1[WALNUT_BRAID][WALNUT_MATRIX_ROW];
		ffelem ansprime[WALNUT_BRAID][WALNUT_MATRIX_ROW];
	} m;

	unsigned char p[WALNUT_BRAID];
	ffelem mp[WALNUT_BRAID];
	ffelem imp[WALNUT_BRAID];
	size_t count;
	unsigned char idx;
	unsigned char leftover;
	unsigned char numbits;
} walnut_verify_ctx;



extern int walnut_verify_init(const void *aekey, const void *dgst,
	walnut_verify_ctx *c);
extern int walnut_verify_update(walnut_verify_ctx *c, const void *sig,
                            size_t len);
extern int walnut_verify_final(walnut_verify_ctx *c);

#endif
